@@ -8,6 +8,8 @@ import tqdm
 
 from . import _gondola_core 
 RBCalibrations = _gondola_core.calibration.RBCalibrations
+RBCalibrations.__module__ = __name__ 
+RBCalibrations.__name__   = 'RBCalibrations'
 
 ## convenience functions
 def load_rb_calibrations(cali_dir : Path, load_event_data = False):
