@@ -197,7 +197,6 @@ pub fn get_califilename(rb_id : u8, latest : bool) -> String {
 }
 
 //----------------------------------------------------------
-
 /// A standardized name for regular run files saved by
 /// the liftof suite
 ///
@@ -210,8 +209,10 @@ pub fn get_califilename(rb_id : u8, latest : bool) -> String {
 ///                a rb id can be specified as well
 /// * timestamp :  substitute the current time with this timestamp
 ///                (or basically any other string) instead.
+/// * tof_only  :  if true, the filename will end with the suffix 
+///                .tof.gaps, if false it will end simply with .gaps 
 #[cfg_attr(feature="pybindings", pyfunction)]
-pub fn get_runfilename(run : u32, subrun : u64, rb_id : Option<u8>, timestamp : Option<String>) -> String {
+pub fn get_runfilename(run : u32, subrun : u64, rb_id : Option<u8>, timestamp : Option<String>, tof_only : bool) -> String {
   let ts : String;
   match timestamp {
     Some(_ts) => {
@@ -224,10 +225,14 @@ pub fn get_runfilename(run : u32, subrun : u64, rb_id : Option<u8>, timestamp :
   let fname : String;
   match rb_id {
     None => {
-      fname = format!("Run{run}_{subrun}.{ts}.gaps");
+      if tof_only {
+        fname = format!("Run{run}_{subrun}.{ts}.tof.gaps"); 
+      } else {
+        fname = format!("Run{run}_{subrun}.{ts}.gaps");
+      }
     }
     Some(rbid) => {
-      fname = format!("Run{run}_{subrun}.{ts}.RB{rbid:02}.gaps");
+      fname = format!("Run{run}_{subrun}.{ts}.RB{rbid:02}.tof.gaps");
     }
   }
   fname
